package envoy.client.ui;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import envoy.data.LoginCredentials;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>LoginDialog.java</strong><br>
 * Created: <strong>01.01.2020</strong><br>
 * 
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @since Envoy v0.3-alpha
 */
public class LoginDialog extends JDialog {

	private final JPanel contentPanel = new JPanel();

	private static final long	serialVersionUID	= 352021600833907468L;
	private JTextField			textField;
	private JPasswordField		passwordField;

	private JPasswordField		repeatPasswordField;
	private JLabel				lblRepeatPassword;
	private GridBagConstraints	gbc_lblRepeatPassword;
	private GridBagConstraints	gbc_repeatPasswordField;

	private LoginCredentials credentials;

	/**
	 * Displays a dialog enabling the user to enter their user name and password.
	 * 
	 * @since Envoy v0.3-alpha
	 */
	public LoginDialog() {
		setSize(338, 123);
		setLocationRelativeTo(null);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		GridBagLayout gbl_contentPanel = new GridBagLayout();
		gbl_contentPanel.columnWidths	= new int[] { 0, 0, 0 };
		gbl_contentPanel.rowHeights		= new int[] { 0, 0, 0 };
		gbl_contentPanel.columnWeights	= new double[] { 0.0, 1.0, Double.MIN_VALUE };
		gbl_contentPanel.rowWeights		= new double[] { 0.0, 0.0, Double.MIN_VALUE };
		contentPanel.setLayout(gbl_contentPanel);
		{
			JLabel				lblUserName		= new JLabel("User name:");
			GridBagConstraints	gbc_lblUserName	= new GridBagConstraints();
			gbc_lblUserName.anchor	= GridBagConstraints.EAST;
			gbc_lblUserName.insets	= new Insets(0, 0, 5, 5);
			gbc_lblUserName.gridx	= 0;
			gbc_lblUserName.gridy	= 0;
			contentPanel.add(lblUserName, gbc_lblUserName);
		}
		{
			textField = new JTextField();
			GridBagConstraints gbc_textField = new GridBagConstraints();
			gbc_textField.insets	= new Insets(0, 0, 5, 0);
			gbc_textField.fill		= GridBagConstraints.HORIZONTAL;
			gbc_textField.gridx		= 1;
			gbc_textField.gridy		= 0;
			contentPanel.add(textField, gbc_textField);
			textField.setColumns(10);
		}
		{
			JLabel				lblPassword		= new JLabel("Password:");
			GridBagConstraints	gbc_lblPassword	= new GridBagConstraints();
			gbc_lblPassword.anchor	= GridBagConstraints.EAST;
			gbc_lblPassword.insets	= new Insets(0, 0, 0, 5);
			gbc_lblPassword.gridx	= 0;
			gbc_lblPassword.gridy	= 1;
			contentPanel.add(lblPassword, gbc_lblPassword);
		}
		{
			passwordField = new JPasswordField();
			GridBagConstraints gbc_passwordField = new GridBagConstraints();
			gbc_passwordField.fill	= GridBagConstraints.HORIZONTAL;
			gbc_passwordField.gridx	= 1;
			gbc_passwordField.gridy	= 1;
			contentPanel.add(passwordField, gbc_passwordField);
		}
		{
			lblRepeatPassword				= new JLabel("Repeat Password:");
			gbc_lblRepeatPassword			= new GridBagConstraints();
			gbc_lblRepeatPassword.anchor	= GridBagConstraints.EAST;
			gbc_lblRepeatPassword.insets	= new Insets(0, 0, 0, 5);
			gbc_lblRepeatPassword.gridx		= 0;
			gbc_lblRepeatPassword.gridy		= 2;
		}
		{
			repeatPasswordField = new JPasswordField();
			gbc_repeatPasswordField			= new GridBagConstraints();
			gbc_repeatPasswordField.fill	= GridBagConstraints.HORIZONTAL;
			gbc_repeatPasswordField.gridx	= 1;
			gbc_repeatPasswordField.gridy	= 2;
		}
		{
			JPanel buttonPane = new JPanel();

			JTextPane registerText = new JTextPane();
			registerText.setText("Register?");
			registerText.setFont(new Font("Arial", Font.BOLD, 12));
			registerText.setAlignmentX(LEFT_ALIGNMENT);
			buttonPane.add(registerText);

			JCheckBox registerCheckBox = new JCheckBox();
			registerCheckBox.setAlignmentX(LEFT_ALIGNMENT);
			registerCheckBox.addItemListener(new ItemListener() {

				@Override
				public void itemStateChanged(ItemEvent e) {
					switch (e.getStateChange()) {
						case ItemEvent.SELECTED:
							contentPanel.add(lblRepeatPassword, gbc_lblRepeatPassword);
							contentPanel.add(repeatPasswordField, gbc_repeatPasswordField);
							setSize(338, 160);
							contentPanel.revalidate();
							contentPanel.repaint();
							break;

						case ItemEvent.DESELECTED:
							if (repeatPasswordField.getParent() == contentPanel) {
								contentPanel.remove(lblRepeatPassword);
								contentPanel.remove(repeatPasswordField);
								setSize(338, 123);
								contentPanel.revalidate();
								contentPanel.repaint();
							}
							break;
					}
				}
			});
			buttonPane.add(registerCheckBox);

			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener((evt) -> {
					try {
						if (registerCheckBox.isSelected()) {
							if (Arrays.equals(passwordField.getPassword(), repeatPasswordField.getPassword())) {
								credentials = new LoginCredentials(textField.getText(), passwordField.getPassword(), true);
								dispose();
							} else {
								JOptionPane.showMessageDialog(this, "The repeated password is unequal to the origional password!");
								passwordField.setText(null);
								repeatPasswordField.setText(null);
							}
						} else {
							credentials = new LoginCredentials(textField.getText(), passwordField.getPassword(), false);
							dispose();
						}
					} catch (NoSuchAlgorithmException e) {
						e.printStackTrace();
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener((evt) -> dispose());
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}

		setModal(true);
		setVisible(true);
	}

	/**
	 * @return the {@link LoginCredentials} entered by the user, or {@code null} if
	 *         the dialog has been cancelled
	 * @since Envoy v0.3-alpha
	 */
	public LoginCredentials getCredentials() { return credentials; }
}